

import org.vertx.java.core.Handler;
import org.vertx.java.core.SimpleHandler;
import org.vertx.java.core.http.HttpClient;
import org.vertx.java.core.http.HttpClientRequest;
import org.vertx.java.core.http.HttpClientResponse;
import org.vertx.java.core.http.HttpServerRequest;
import org.vertx.java.core.streams.Pump;
import org.vertx.java.deploy.Verticle;

public class ProxyServer extends Verticle {
   private static final int PROXY_PORT = 8080;
   private static final int SERVER_PORT = 8282;

   @Override
   public void start() {
      System.out.println("Started proxy on port " + PROXY_PORT + ". Real Server's PORT is: " + SERVER_PORT);
      //proxy client
      final HttpClient pClient = vertx.createHttpClient().setHost("localhost").setPort(SERVER_PORT);
      //proxy server, intercepting requests from the real client
      vertx.createHttpServer().requestHandler(new Handler<HttpServerRequest>() {
         @Override
         public void handle(final HttpServerRequest psReq) {
            System.out.println("Proxy gets (from client) request: " + psReq.uri);
            
            psReq.pause();
            HttpClientRequest pcReq = pClient.request(psReq.method, psReq.uri, new Handler<HttpClientResponse>() {
               @Override
               public void handle(HttpClientResponse pcResp) {
                  System.out.println("Proxy gets (from server) response: " + pcResp.statusCode);
                  //forward the response to the client
                  psReq.response.statusCode = pcResp.statusCode;
                  psReq.response.headers().putAll(pcResp.headers());
                  psReq.response.end();
               }
            });
            
            //forward the client request to the real server
            pcReq.headers().putAll(psReq.headers());
            pcReq.setChunked(true);
            Pump pump = Pump.createPump(psReq, pcReq);
            pump.start();
            psReq.resume();
         }
      }).listen(PROXY_PORT);
   }
}
