

import java.io.File;
import java.util.UUID;
import org.vertx.java.core.AsyncResult;
import org.vertx.java.core.AsyncResultHandler;
import org.vertx.java.core.Handler;
import org.vertx.java.core.SimpleHandler;
import org.vertx.java.core.file.AsyncFile;
import org.vertx.java.core.http.HttpServerRequest;
import org.vertx.java.core.streams.Pump;
import org.vertx.java.deploy.Verticle;

/**
 *
 */
public class UploadServer extends Verticle {

   /**
    * Need to remove this field from here into config.
    */
   public static final String BLOB_REPOSITORY = "/home/max/Downloads/BLOB_REPOSITORY_TEMP/";
   private static final int PORT = 8282;

   @Override
   public void start() {

      if (!new File(BLOB_REPOSITORY).exists()) {
         throw new IllegalArgumentException("File " + BLOB_REPOSITORY + " does not exist");
      }
      System.out.println("Started server on port " + PORT);
      vertx.createHttpServer().requestHandler(new Handler<HttpServerRequest>() {
         @Override
         public void handle(final HttpServerRequest req) {

            //We first pause the request so we don't receive any data between now and when the file is opened
            req.pause();
            System.out.println("Got request " + req.uri);
            final String filename = BLOB_REPOSITORY + UUID.randomUUID().toString() + ".upload";

            vertx.fileSystem().open(filename, new AsyncResultHandler<AsyncFile>() {
               @Override
               public void handle(AsyncResult<AsyncFile> ar) {
                  final AsyncFile asyncFile = ar.result;
                  final Pump pump = Pump.createPump(req, asyncFile.getWriteStream());

                  final long start = System.currentTimeMillis();
                  req.endHandler(new SimpleHandler() {
                     @Override
                     public void handle() {
                        asyncFile.close(new AsyncResultHandler<Void>() {
                           @Override
                           public void handle(AsyncResult<Void> ar) {
                              if (ar.exception == null) {
                                 req.response.statusCode = 200;
                                 req.response.headers().putAll(req.headers());
                                 req.response.end();
                                 long end = System.currentTimeMillis();
                                 System.out.println("\n\t\tUploaded " + pump.getBytesPumped() + " bytes\n\t\tto "
                                         + filename + "\n\t\tin " + (end - start) + " ms");
                              } else {
                                 ar.exception.printStackTrace(System.err);
                              }
                           }
                        });
                     }
                  });
                  pump.start();
                  req.resume();
               }
            });
         }
      }).listen(PORT);
   }
}
